<?php

/**
 * @package MachII
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
/**
 * Functions for backwards compatibility.
 */
require_once 'MachII/util/BackwardCompat.php';
/**
 * Class Loader
 */
require_once 'MachII/util/ClassLoader.php';
/**
 * Application Loader
 */
require_once 'MachII/framework/AppLoader.php';
/**
 * Application Factory
 */
require_once 'MachII/framework/AppFactory.php';

/**#@+
 * Configuration Reload Codes
 */
/**
 * Never Reload
 * 
 * Always use the cached configuration.  Only reload the configuration
 * file if {@link MachII_framework_AppLoader} is being recreated.
 */
define('MACHII_CONFIG_RELOAD_NEVER',  -1);
/**
 * Dynamic Reload
 * 
 * Reloads the configuration if it has changed since it was last cached.
 */
define('MACHII_CONFIG_RELOAD_DYNAMIC', 0);
/**
 * Always Reload
 * 
 * Will reload the configuration file on each request.
 */
define('MACHII_CONFIG_RELOAD_ALWAYS',  1);
/**#@-*/

/**#@+
 * Event Access Codes
 * 
 * Set via the Mach-II configuration file.
 */
/**
 * Flag as an internal event, one that can only be called by Mach-II.
 */
define('MACHII_EVENT_ACCESS_PRIVATE', 0);
/**
 * Flag as a public event, one that may be called by a user.
 */
define('MACHII_EVENT_ACCESS_PUBLIC',  1);
/**#@-*/

/**#@+
 * Error Codes
 */
define('MACHII_OK',                               1);
define('MACHII_ERROR',                           -1);
define('MACHII_ERROR_IS_PROCESSING_EVENTS',      -2);
define('MACHII_ERROR_EVENT_HANDLER_NOT_DEFINED', -3);
define('MACHII_ERROR_EVENT_IS_PRIVATE',          -4);
define('MACHII_ERROR_EVENT_FILTER_NOT_DEFINED',  -5);
define('MACHII_ERROR_LISTENER_NOT_DEFINED',      -6);
define('MACHII_ERROR_PLUGIN',                    -7);
define('MACHII_ERROR_VIEW_NOT_DEFINED',          -8);
define('MACHII_ERROR_MAX_EVENTS_EXCEEDED',       -9);
define('MACHII_ERROR_LISTENER_DISPATCH_MAP_NOT_DEFINED', -10);
define('MACHII_ERROR_LISTENER_ARG_COUNT_MISMATCH',       -11);
define('MACHII_ERROR_ABORT_EVENT',              -12);
define('MACHII_ERROR_CUSTOM',                   -13);
define('MACHII_ERROR_ABORT_EVENT_EXCEPTION',    -14);
define('MACHII_ERROR_FILTER',                   -15);
define('MACHII_ERROR_CACHE_INIT',               -16);
define('MACHII_ERROR_CONFIG_LOAD',              -17);
define('MACHII_ERROR_CONFIG_PARSE',             -18);
/**#@-*/

/**
 * Container for framework helpers
 * @global array $GLOBALS['_MachII']
 * @name $_MachII
 * @access private
 */
$GLOBALS['_MachII'] = array();

/**
 * MachII provides a convenient interface to lower-level framework
 * classes as well as some globally useful functions.
 * @package MachII
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII
{
    /**
     * Factory for creating a configured application loader.
     * 
     * In general this method should not be called directly; however,
     * it is useful when implementing custom caching mechanisms and
     * other higher level {@link MachII_framework_AppLoader} handlers. 
     * 
     * MachII::appLoader() is called as follows:
     * <code>
     * $appLoader =& MachII::appLoader($configPath, $configMode);
     * if (MachII::isError($appLoader)) {
     *     // error handling
     * }
     * </code>
     * 
     * @static
     * @uses MachII_framework_AppLoader
     * @param string the path and filename of the Mach-II configuration file
     * @param integer the configuration reload mode
     * @return MachII_framework_AppLoader
     * @throws MACHII_ERROR
     */
    function &appLoader($configPath, $configMode)
    {
        // Create the AppLoader
        $appLoader =& new MachII_framework_AppLoader();
        $appLoader->init($configPath);
        
        return $appLoader;
    }
    
    /**
     * Factory for creating a caching application loader.
     * 
     * It is strongly recommended that some form of caching be used
     * with Mach-II.  Due to the way it functions the entire application
     * must be present when processing an event request.  There is a
     * relatively high overhead in preparing the application for use.
     * The use of a cache prevents the recurrance of that overhead by
     * reusing the prepared application.
     * In its simplest form MachII::cache() only requires three parameters:
     * <code>
     * $appLoader =& MachII::cache(
     *     $MACHII_CONFIG_PATH,
     *     $MACHII_CONFIG_MODE,
     *     $MACHII_APP_KEY
     * );
     * if (MachII::isError($appLoader)) {
     *     // error handling
     * }
     * </code>
     * This will use MachII's built in user session cache,
     * {@link MachII_cache_PhpSession}.  It is <b>not</b> intended
     * for high volume applications.  For better scalability, use a
     * cache system that allows a single {@link MachII_framework_AppLoader}
     * to be shared by multiple users.
     * 
     * Note: MachII_cache_PhpSession will fail silently on PHP <4.1.0.
     * The application will NOT be cached!  Use an alternate cache.
     * 
     * To use another cache system, install the driver in
     * MachII/cache/ and call it by its 'short name'.  Any
     * cache driver specific options or default overrides may be passed as
     * an associative array.
     * <code>
     * $appLoader =& MachII::cache(
     *     $MACHII_CONFIG_PATH,
     *     $MACHII_CONFIG_MODE,of
     *     $MACHII_APP_KEY,
     *     // use a cache driver that uses PEAR::Cache_Lite for storage
     *     'PearCacheLite', 
     *     array(
     *         // override the cache directory
     *         'cacheDir' => '/path/to/cache/dir/',
     *         // change the cache life time to 2 days
     *         'lifeTime' => 172800
     *     )
     * );
     * </code>
     * 
     * @tutorial MachII/configphp.pkg#machiiphp.cache
     * @static
     * @uses MachII::appLoader()
     * @internal uses {@link MachII::_cache()}
     * @internal uses {@link $_MachII}
     * @uses MachII_framework_Cache
     * @uses MachII_cache_PhpSession (by default)
     * @global array $_MachII
     * @param string the path and filename to the Mach-II configuration file
     * @param integer the configuration reload mode
     * @param string a unique name for this Mach-II application
     * @param string a valid container for caching the configured application
     * @param array the cache container's configuration options
     * @return MachII_framework_AppLoader
     * @throws MACHII_ERROR_CACHE_INIT
     * @throws MACHII_ERROR*
     */
    function &cache($configPath, $configMode, $name, $container = 'MachII.cache.PhpSession', $options = array())
    {
        // Get cache ready
        MachII_util_ClassLoader::load('MachII.framework.Cache');
        $cachePlugin = MachII_util_ClassLoader::load($container);
        $GLOBALS['_MachII']['cache'] =& new $cachePlugin($name, $options);
        $cache =& $GLOBALS['_MachII']['cache'];
        if (MachII::isError($cache)) {
            $error =& MachII::throwError(
                array('container' => $container),
                MACHII_ERROR_CACHE_INIT
            );
            return $error;
        }
        
        
        $appLoader = false;
        
        // Restore the cached AppManager
        if ($classes = $cache->get('MachII_Classes')) {
            MachII_util_ClassLoader::restore($classes);
            $appLoader = $cache->get('MachII_AppLoader');
            //echo 'restored appLoader<br />';
        }
        
        // Or build a new AppManager
        if (($classes === false)
        || ($appLoader === false)) {
            $cache->clear();
            $appLoader =& MachII::appLoader($configPath, $configMode);
            if (MachII::isError($appLoader)) {
                return $appLoader;
            }
            //echo 'built new appLoader<br />';
        } else {
            // Reload the configuration
            if (($configMode == MACHII_CONFIG_RELOAD_ALWAYS)
            || (($configMode == MACHII_CONFIG_RELOAD_DYNAMIC)
              && ($appLoader->shouldReloadConfig()))) {
                $cache->clear();
                $error = $appLoader->reloadConfig();
                if (MachII::isError($error)) {
                    return $error;
                }
                //echo 'reloaded config<br />';
            }
        }
        
        // For later caching
        $GLOBALS['_MachII']['appLoader'] =& $appLoader;
        
        // Setup shutdown
        register_shutdown_function(array('MachII', '_cache'));
        
        return $appLoader;
    }
    
    /**
     * Shutdown method for caching application loader.
     * 
     * @static
     * @access private
     */
    function _cache()
    {
        if (isset($GLOBALS['_MachII']['cache'])) {
            $cache =& $GLOBALS['_MachII']['cache'];
            
            // AppManager
            if ((isset($GLOBALS['_MachII']['appLoader']))
            && (is_a($GLOBALS['_MachII']['appLoader'], 'MachII_framework_AppLoader'))) {
            //echo 'caching appLoader';
                $appMgrSaved = $cache->save('MachII_AppLoader', $GLOBALS['_MachII']['appLoader']);
            }
            
            // Class file paths
            $classes =& MachII_util_ClassLoader::singleton();
            if (is_array($classes)) {
                $classesSaved = $cache->save('MachII_Classes', $classes);
            }
        }
    }
    
    /**
     * Application programming interface version number
     * 
     * Currently this is set to the version number of the analogous reference
     * implementation, Mach-II for ColdFusion.
     * 
     * @static
     * @return string version number
     */
    function apiVersion()
    {
        return '1.0.9';
    }
    
    /**
     * Package version number
     * 
     * @static
     * @return string version number
     */
    function version()
    {
        return '0.3.14-alpha';
    }
    
    /**
     * Wrapper for error class.
     * 
     * Loads error class on demand and creates an error object.
     * 
     * Currently uses PEAR_Error.  If PEAR is available in the
     * include path that one will be used, otherwise the bundled
     * version will be loaded.
     * 
     * @param string a brief message providing context for the error
     * @param integer an error code
     * @return PEAR_Error
     */
    function &throwError($message, $code = MACHII_ERROR)
    {
        // load PEAR
        if ((!class_exists('PEAR_Error'))// already loaded
        && (!@include_once 'PEAR.php')) {// in PEAR include path
            // load bundled copy
            require_once 'MachII/util/pear/PEAR.php';
        }
        
        $error =& PEAR::raiseError($message, $code);
        
        return $error;
    }
    
    /**
     * @param MACHII_ERROR*
     * @return MachII_util_Exception
     */
    function &createException($error)
    {
        $className = MachII_util_ClassLoader::load('MachII.util.Exception');
        $exception =& new $className($error);
        
        return $exception;
    }
    
    /**
     * Check if a value is an error object.
     * 
     * @static
     * @param mixed a value to check
     * @return boolean
     */
    function isError($error)
    {
        return is_a($error, 'PEAR_Error');
    }
    
    /**
     * Retrieve the rendered error message from an error code and message
     * or from an error object.
     *
     * @static
     * @param integer an error code
     * @param array associative array of message replacement values
     * @return string the complete error message
     */
    function errorMessage($value, $message = array())
    {
        static $errorMessages;
        $localePath    = 'MachII/locale/';
        $defaultLocale = 'en';
        $locale        = $defaultLocale;
        
        $appLoader   =& $GLOBALS['_MachII']['appLoader'];
        $appManager  =& $appLoader->getAppManager();
        $propManager =& $appManager->getPropertyManager();
        if ($propManager->hasProperty('locale')) {
            $locale = $propManager->getProperty('locale');
        }
        
        if (!isset($errorMessages)) {
        
            if ((!preg_match('/^[a-z]{2}(_[A-Z]{2}|)$/', $locale))
            || (!file_exists($localePath.$locale.'.php'))) {
                $locale = $defaultLocale;
            }
        
            include $localePath.$locale.'.php';
        }
        
        if (isset($errorMessages[$value])) {
            $renderedMsg = $errorMessages[$value];
            if (is_string($message)) {
                $message = array('message' => $message);
            }
            foreach ($message as $name=>$replacement) {
                $renderedMsg = str_replace('{'.$name.'}', $replacement, $renderedMsg);
            }
        } else {
            $renderedMsg = $errorMessages[MACHII_ERROR];
        }
        
        return $renderedMsg;
    }

}

?>