<?php
/**
 * @package MachII
 */
/**
 * Load Plugin base class.
 */
require_once 'MachII/framework/Plugin.php';
/**
 * Manages registered plugins.
 * @package MachII
 */
class MachII_framework_PluginManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    var $_appManager;
    
    /**
     * @var array associative array of MachII_framework_Plugin descendents.
     * @access private
     */
    var $_plugins = array();
    
    
    /**
     * @param array associative array of plugin configuration data.
     * @param MachII_framework_AppManager
     */
    function init($plugins, &$appManager)
    {
        $this->setAppManager($appManager);
        
        foreach ($plugins as $name=>$att) {
            $pluginType = MachII_util_ClassLoader::load($att['type']);
            $plugin =& new $pluginType($appManager, $att['parameters']);
            
            $this->addPlugin($name, $plugin);
        }
    }
    
    /**
     * Configure all of the plugins.
     * @throws MACHII_ERROR*
     */
    function configure()
    {
        $keys = array_keys($this->_plugins);
        foreach ($keys as $key) {
            $plugin =& $this->getPlugin($key);
            if (MachII::isError($plugin)) {
                return $listener;
            }
            
            $error = $plugin->configure();
            if (MachII::isError($error)) {
                return $error;
            }
        }
    }
    
    /**
     * @param string
     * @param MachII_framework_Plugin descendent
     */
    function addPlugin($pluginName, &$plugin)
    {
        $this->_plugins[$pluginName] =& $plugin;
    }
    
    /**
     * @param string
     * @return MachII_framework_Plugin descendent
     */
    function &getPlugin($pluginName)
    {
        return $this->_plugins[$pluginName];
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    function setAppManager(&$appManager)
    {
        $this->_appManager =& $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    function &getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param string method name of a plugin point
     * @param MachII_framework_EventContext
     * @throws MACHII_ERROR_PLUGIN, MACHII_ERROR*
     */
    function executePluginPoint($pluginPoint, &$eventContext)
    {
        $keys = array_keys($this->_plugins);
        foreach ($keys as $key) {
            $plugin =& $this->getPlugin($key);
            $error = $plugin->$pluginPoint($eventContext);
            if (MachII::isError($error)) {
                return $error;
            } elseif ($error === false) {
                $error =& MachII::throwError(
                    array(
                        'className'  => get_class($plugin),
                        'methodName' => $pluginPoint
                    ),
                    MACHII_ERROR_PLUGIN
                );
                return $error;
            }
        }
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    function preProcess(&$eventContext)
    {
        $this->executePluginPoint('preProcess', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    function preEvent(&$eventContext)
    {
        $this->executePluginPoint('preEvent', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    function postEvent(&$eventContext)
    {
        $this->executePluginPoint('postEvent', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    function preView(&$eventContext)
    {
        $this->executePluginPoint('preView', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    function postView(&$eventContext)
    {
        $this->executePluginPoint('postView', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    function postProcess(&$eventContext)
    {
        $this->executePluginPoint('postProcess', $eventContext);
    }
    
    /**
     * Executes handleException() for each registered plugin.
     * @param MachII_framework_EventContext
     * @param MachII_framework_Event exception event
     * @see MachII_framework_EventContext::handleException()
     */
    function handleException(&$eventContext, &$exception)
    {
        $keys = array_keys($this->_plugins);
        foreach ($keys as $key) {
            $plugin =& $this->getPlugin($key);
            $plugin->handleException($eventContext, $exception);
        }
    }
}
 
?>