<?php
/**
 * @package MachII
 */

require_once 'MachII/framework/EventHandler.php';
require_once 'MachII/framework/EventCommand.php';

/**
 * Manages registered EventHandlers for the framework.
 * @package MachII
 */
class MachII_framework_EventManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var array associative array of event handlers
     * @access private
     */
    private $_handlers = array();
    
    /**
     * @var MachII_framework_ListenerManager temporary
     * @access private
     */
    private $_listenerMgr;
    
    /**
     * @var MachII_framework_FilterManager temporary
     * @access private
     */
    private $_filterMgr;
    
    
    /**
     * @param SimpleXMLElement
     * @param MachII_framework_AppManager
     * @throws MACHII_ERROR*
     */
    public function __construct(SimpleXMLElement $configXml, MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
        
        $this->_listenerMgr = $appManager->getListenerManager();
        $this->_filterMgr   = $appManager->getFilterManager();
        
        $events = $configXml->xpath('//event-handlers/event-handler');
        if (empty($events)) {
            break;
        }
        foreach ($events as $event) {
            $eventName = strtolower((string) $event['event']);
            $eventHandler = new MachII_framework_EventHandler();
            $eventAccess = (isset($event['access'])
                && (strtolower((string) $event['access']) == 'private'))?
                    MACHII_EVENT_ACCESS_PRIVATE:
                    MACHII_EVENT_ACCESS_PUBLIC;
            
            $eventHandler->setAccess($eventAccess);
            
            $this->_handlers[$eventName] = $eventHandler;
            
            foreach ($event->children() as $commandType => $command) {
                $eventCommand = $this->_createEventCommand($commandType, $command);
                $eventHandler->addCommand($eventCommand);
            }
        }
        
        unset($this->_listenerMgr);
        unset($this->_filterMgr);
    }
    
    /**
     * @internal doesn't do anything at this time
     */
    public function configure() {}

    /**
     * @param string
     * @param array associative array of event arguments
     * @param string
     * @throws MACHII_ERROR_EVENT_HANDLER_NOT_DEFINED
     * @return MachII_framework_Event or descendent
     */
    public function createEvent(
        $eventName,
        $eventArgs = array(),
        $eventType = 'MachII.framework.Event')
    {
        $eventName = strtolower($eventName);
        if (!$this->isEventDefined($eventName)) {
            $error = MachII::throwError(
                array('eventName' => $eventName),
                MACHII_ERROR_EVENT_HANDLER_NOT_DEFINED);
            return $error;
        }
        
        $eventType = MachII_util_ClassLoader::load($eventType);
        
        $event = new $eventType($eventName, $eventArgs);
        
        return $event;
    }
    
    /**
     * @param string
     * @throws MACHII_ERROR_EVENT_HANDLER_NOT_DEFINED
     * @return MachII_framework_EventHandler
     */
    public function getEventHandler($eventName)
    {//echo 'getting event handler: ',$eventName,'<br />';
        $eventName = strtolower($eventName);
        if (!$this->isEventDefined($eventName)) {
            $error = MachII::throwError(
                array('eventName' => $eventName),
                MACHII_ERROR_EVENT_HANDLER_NOT_DEFINED);
            return $error;
        }//echo 'handler exists<br />';
        $eventHandler = $this->_handlers[$eventName];
        
        return $eventHandler;
    }
    
    /**
     * @param string
     * @return boolean
     */
    public function isEventDefined($eventName)
    {
        $eventName = strtolower($eventName);
        
        return isset($this->_handlers[$eventName]);
    }
    
    /**
     * @param string
     * @return boolean
     */
    public function isEventPublic($eventName)
    {
        $eventName = strtolower($eventName);
        
        return ($this->getEventHandler($eventName)->getAccess() == MACHII_EVENT_ACCESS_PUBLIC);
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * Creates the appropriate event command from a raw event configuration.
     * @param string MachII_framework_EventCommand type
     * @param SimpleXMLElement event commands
     * @access private
     */
    private function _createEventCommand($commandType, SimpleXMLElement $command)
    {
        switch ($commandType) {
            case 'filter':
                $parameters = (isset($command->parameters->parameter))?
                    $command->parameters->parameter:
                    array();
                    
                $filterParams = array();
                foreach ($parameters as $parameter) {
                    $filterParams[(string) $parameter['name']] = (string) $parameter['value'];
                }
                
                $filter = $this->_filterMgr->getFilter((string) $command['name']);
                
                $cmd = MachII_util_ClassLoader::load('MachII.framework.commands.FilterCommand');
                $eventCommand = new $cmd($filter, $filterParams);
                break;
                
            case 'event-arg':
                $value = isset($command['value'])?
                    (string) $command['value']:
                    null;
                $variable = isset($command['variable'])?
                    (string) $command['variable']:
                    null;
                    
                $cmd = MachII_util_ClassLoader::load('MachII.framework.commands.EventArgCommand');
                $eventCommand = new $cmd((string) $command['name'], $value, $variable);
                break;
                
            case 'event-mapping':
                $cmd = MachII_util_ClassLoader::load('MachII.framework.commands.EventMappingCommand');
                $eventCommand = new $cmd((string) $command['event'], (string) $command['mapping']);
                break;
                
            case 'notify':
                $resultKey = isset($command['resultKey'])?
                    (string) $command['resultKey']:
                    null;
                
                $listener = $this->_listenerMgr->getListener((string) $command['listener']);
                
                $cmd = MachII_util_ClassLoader::load('MachII.framework.commands.NotifyCommand');
                $eventCommand = new $cmd($listener, (string) $command['method'], $resultKey);
                break;
                
            case 'announce':
                $copyEventArgs = (isset($command['copyEventArgs'])
                    && ((string) $command['copyEventArgs'] == 'false'))?
                        false:true;
                        
                $cmd = MachII_util_ClassLoader::load('MachII.framework.commands.AnnounceCommand');
                $eventCommand = new $cmd((string) $command['event'], (string) $command['copyEventArgs']);
                break;
                
            case 'view-page':
                $contentKey = isset($command['contentKey'])?
                    (string) $command['contentKey']:
                    null;
                    
                $cmd = MachII_util_ClassLoader::load('MachII.framework.commands.ViewPageCommand');
                $eventCommand = new $cmd((string) $command['name'], (string) $contentKey);
                break;
                
            case 'event-bean':
                $fields = isset($command['fields'])?
                    (string) $command['fields']:
                    null;
                    
                $cmd = MachII_util_ClassLoader::load('MachII.framework.commands.EventBeanCommand');
                $eventCommand = new $cmd((string) $command['name'], (string) $command['type'], $fields);
                break;
                
            default:
                $cmd = MachII_util_ClassLoader::load('MachII.framework.EventCommand');
                $eventCommand = new $cmd();
                break;
        }// end switch $command
        
        return $eventCommand;
    }

}

?>