<?php
/**
 * @package MachII
 */
/**
 * Load base event filter class
 */
require_once 'MachII/framework/EventFilter.php';
/**
 * Manages registered {@link MachII_framework_EventFilter}s.
 * @package MachII
 */
class MachII_framework_FilterManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var array associative array of MachII_framework_EventFilter descendents
     * @access private
     */
    private $_filters = array();
    
    
    /**
     * @param SimpleXMLElement
     * @param MachII_framework_AppManager
     */
    public function __construct(SimpleXMLElement $configXml, MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
        
        $filters = $configXml->xpath('//event-filters/event-filter');
        if (empty($filters)) {
            return;
        }
        foreach ($filters as $filter) {
            $parameters = isset($filter->parameters->parameter)?
                $filter->parameters->parameter:
                array();
            $filterParams = array();
            foreach ($parameters as $param) {
                $filterParams[(string) $param['name']] = (string) $param['value'];
            }
            
            $filterType = MachII_util_ClassLoader::load((string) $filter['type']);
            
            $this->addFilter($filter['name'], new $filterType($appManager, $filterParams));
        }
    }
    
    /**
     * Configure all of the filters
     * @throws MACHII_ERROR*
     */
    public function configure()
    {
        $keys = array_keys($this->_filters);
        foreach ($keys as $key) {
            $filter = $this->getFilter($key);
            if (MachII::isError($filter)) {
                return $filter;
            }
            
            $error = $filter->configure();
            if (MachII::isError($error)) {
                return $error;
            }
        }
    }
    
    /**
     * @param string
     * @param MachII_framework_EventFilter or descendent
     */
    public function addFilter($name, MachII_framework_EventFilter $filter)
    {
        $this->_filters[$name] = $filter;
    }
    
    /**
     * @param string
     * @throws MACHII_ERROR_FILTER_NOT_DEFINED
     * @return MachII_framework_EventFilter or descendent
     */
    public function getFilter($filterName)
    {
        if (!$this->isFilterDefined($filterName)) {
            $error = MachII::throwError(
                array('filterName' => $filterName),
                MACHII_ERROR_FILTER_NOT_DEFINED);
            return $error;
        }
        
        return $this->_filters[$filterName];
    }
    
    /**
     * @param string
     * @return MachII_framework_EventFilter or descendent
     */
    public function isFilterDefined($filterName)
    {
        return isset($this->_filters[$filterName]);
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }

}

?>