<?php
/**
 * @package MachII
 */
/**
 * Load base plugin class
 */
require_once 'MachII/framework/Plugin.php';
/**
 * Manages registered plugins.
 * @package MachII
 */
class MachII_framework_PluginManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var array associative array of MachII_framework_Plugin descendents.
     * @access private
     */
    private $_plugins = array();
    
    
    /**
     * @param SimpleXMLElement
     * @param MachII_framework_AppManager
     */
    public function __construct(SimpleXMLElement $configXml, MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
        
        $plugins = $configXml->xpath('//plugins/plugin');
        if (empty($plugins)) {
            return;
        }
        
        foreach ($plugins as $plugin) {
            $parameters = isset($plugin->parameters->parameter)?
                $filter->parameters->parameter:
                array();
                
            $pluginParams = array();
            foreach ($parameters as $param) {
                $filterParams[(string) $param['name']] = (string) $param['value'];
            }
            
            $pluginType = MachII_util_ClassLoader::load((string) $plugin['type']);
            $plugin = new $pluginType($appManager, $pluginParams);
            
            $this->addPlugin((string) $plugin['name'], $plugin);
        }
    }
    
    /**
     * Configure all of the plugins.
     * @throws MACHII_ERROR*
     */
    public function configure()
    {
        $keys = array_keys($this->_plugins);
        foreach ($keys as $key) {
            $plugin =& $this->getPlugin($key);
            if (MachII::isError($plugin)) {
                return $listener;
            }
            
            $error = $plugin->configure();
            if (MachII::isError($error)) {
                return $error;
            }
        }
    }
    
    /**
     * @param string
     * @param MachII_framework_Plugin descendent
     */
    public function addPlugin($pluginName, MachII_framework_Plugin $plugin)
    {
        $this->_plugins[$pluginName] = $plugin;
    }
    
    /**
     * @param string
     * @return MachII_framework_Plugin descendent
     */
    public function getPlugin($pluginName)
    {
        return $this->_plugins[$pluginName];
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param string method name of a plugin point
     * @param MachII_framework_EventContext
     * @throws MACHII_ERROR_PLUGIN, MACHII_ERROR*
     */
    public function executePluginPoint($pluginPoint, MachII_framework_EventContext $eventContext)
    {
        $keys = array_keys($this->_plugins);
        foreach ($keys as $key) {
            $plugin = $this->getPlugin($key);
            $error = $plugin->$pluginPoint($eventContext);
            if (MachII::isError($error)) {
                return $error;
            } elseif ($error === false) {
                $error =& MachII::throwError(
                    array(
                        'className'  => get_class($plugin),
                        'methodName' => $pluginPoint
                    ),
                    MACHII_ERROR_PLUGIN
                );
                return $error;
            }
        }
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function preProcess(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('preProcess', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function preEvent(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('preEvent', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function postEvent(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('postEvent', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function preView(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('preView', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function postView(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('postView', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function postProcess(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('postProcess', $eventContext);
    }
    
    /**
     * Executes handleException() for each registered plugin.
     * @param MachII_framework_EventContext
     * @param MachII_framework_Exception
     * @see MachII_framework_EventContext::handleException()
     */
    public function handleException(MachII_framework_EventContext $eventContext, MachII_util_Exception $exception)
    {
        $keys = array_keys($this->_plugins);
        foreach ($keys as $key) {
            $this->getPlugin($key)->handleException($eventContext, $exception);
        }
    }
}
 
?>