<?php
/**
 * @package MachII
 */
/**
 * Responsible for controlling loading/reloading of the AppManager.
 * @package MachII
 */
class MachII_framework_AppLoader
{
    /**
     * Path and filname of the XML configuration file
     * @access private
     * @var string
     */
    private $_configPath;
    /**
     * Application manager
     * @access private
     * @var MachII_framework_AppManager
     */
    private $_appManager;
    /**
     * Application factory
     * @access private
     * @var MachII_framework_AppFactory
     */
    private $_appFactory;
    /**
     * Date the configuration file was last reloaded.
     * 
     * @access private
     * @var integer Unix timestamp
     */
    private $_lastReloadDate = 0;
    
    
    /**
     * Configure the application
     * @param string path and filename of the configuration file
     */
    public function __construct($configPath)
    {
        $appFactory = new MachII_framework_AppFactory();
        $this->setAppFactory($appFactory);
        $this->setConfigPath($configPath);
        $this->reloadConfig();
    }
    
    /**
     * Determine if the configuration file should be reloaded
     * 
     * Checks if the configuration file has been modified since
     * it was last loaded.
     * @return boolean
     */
    public function shouldReloadConfig()
    {
        $shouldReload = ($this->getLastReloadDate() < filemtime($this->getConfigPath()));
        
        return $shouldReload;
    }
    
    /**
     * Create a new application manager
     * 
     * Creates a new application manager from the configuration
     * and updates the last reload date to the current date/time.
     */
    public function reloadConfig()
    {
        $appManager = $this->getAppFactory()->createAppManager($this->getConfigPath());
        
        $this->setAppManager($appManager);
        //echo '<pre>',var_dump($appManager->getPropertyManager()),'</pre>';
        $this->setLastReloadDate(time());
    }
    
    /**
     * @param integer Unix timestamp
     */
    public function setLastReloadDate($lastReloadDate)
    {
        $this->_lastReloadDate = $lastReloadDate;
    }
    
    /**
     * @return integer Unix timestamp
     */
    public function getLastReloadDate()
    {
        return $this->_lastReloadDate;
    }
    
    /**
     * Set the path and filename of the configuration file
     * @param string absolute path and filename of the configuration file
     */
    public function setConfigPath($configPath)
    {
        $this->_configPath = $configPath;
    }
    
    /**
     * @return string path and filename of the configuration file
     */
    public function getConfigPath()
    {
        return $this->_configPath;
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param MachII_framework_AppFactory
     */
    public function setAppFactory(MachII_framework_AppFactory $appFactory)
    {
        $this->_appFactory = $appFactory;
    }
    
    /**
     * @return MachII_framework_AppFactory
     */
    public function getAppFactory()
    {
        return $this->_appFactory;
    }

}

?>