<?php
/**
 * @package MachII
 */
/**
 * Handles request to event conversion.
 * @package MachII
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_RequestHandler
{
    /**
     * @var MachII_framework_AppManager
     * @private
     */
    private $_appManager;
    
    
    /**
     * @param MachII_framework_AppManager
     */
    public function __construct(MachII_framework_AppManager $appManager)
    {
        $this->_setAppManager($appManager);
    }
    
    /**
     * Entry point for handling an incoming request.
     * @uses getRequestEventArgs()
     * @uses getEventName()
     * @uses handleEventRequest()
     */
    public function handleRequest()
    {//echo '<pre>',var_dump($this->_appManager->getPropertyManager()),'</pre>';
        $eventArgs =& $this->_getRequestEventArgs();
        $eventName = $this->_getEventName($eventArgs);
        
        $error = $this->handleEventRequest($eventName, $eventArgs);
    }
    
    /**
     * Handles the incoming event request.
     * 
     * Will announce the event and start event processing.
     * @uses MachII_framework_EventContext::announceEvent()
     * @uses MachII_framework_EventContext::processEvents()
     * @throws MachII_util_EventHandlerNotDefinedException
     * @throws MachII_util_EventIsPrivateException
     */
    public function handleEventRequest($eventName, &$eventArgs = array())
    {
        $appManager = $this->_getAppManager();
        $eventManager = $appManager->getEventManager();
        
        $eventContext = $appManager->createEventContext();
        $GLOBALS['eventContext'] = $eventContext;
        
        try {
            if (!$eventManager->isEventDefined($eventName)) {
                throw new MachII_util_EventHandlerNotDefinedException(array('eventName' => $eventName));
            }
            
            if ($eventManager->isEventPublic($eventName)) {
                // Announce the event
                $eventContext->announceEvent($eventName, $eventArgs);
            } else {
                throw new MachII_util_EventIsPrivateException(array('eventName' => $eventName));
            }
            
        // Handle any errors with the exception event
        } catch (Exception $e) {
            $eventContext->handleException($e, true);
        }
        
        // Start the event processing
        $error = $eventContext->processEvents();
    }
    
    /**
     * @access private
     * @param array associative array of event arguments.
     * @return string
     */
    private function _getEventName($eventArgs)
    {
        $propertyManager = $this->_getAppManager()->getPropertyManager();
        
        $eventParam = $propertyManager->getProperty('eventParameter');
        
        $eventName = (isset($eventArgs[$eventParam]))?
            $eventArgs[$eventParam]:
            $propertyManager->getProperty('defaultEvent');
        $eventName = strtolower($eventName);
        
        return $eventName;
    }
    
    /**
     * Merges global arrays into an associative array according to
     * the configured "parameterPrecedence".
     * 
     * @access private
     * @return array asociative array of event arguments
     */
    private function &_getRequestEventArgs()
    {
        $varOrder = strtolower($this->_getRequestEventVariablesOrder());
        
        $eventArgs = array();
        
        $varOrderLen = strlen($varOrder);
        for ($i = 0; $i < $varOrderLen; $i++) {
            switch ($varOrder{$i}) {
                case 'e':
                    $eventArgs =& array_merge($eventArgs, $GLOBALS['_ENV']);
                    break;
                case 'g':
                    $eventArgs =& array_merge($eventArgs, $GLOBALS['_GET']);
                    break;
                case 'p':
                    $eventArgs =& array_merge($eventArgs, $GLOBALS['_POST']);
                    break;
                case 'c':
                    $eventArgs =& array_merge($eventArgs, $GLOBALS['_COOKIE']);
                    break;
                case 's':
                    $eventArgs =& array_merge($eventArgs, $GLOBALS['_SERVER']);
                    break;
                case 'r':
                    $eventArgs =& $GLOBALS['_REQUEST'];
                    break;
                case 'f':
                    $eventArgs =& array_merge($eventArgs, $GLOBALS['_FILES']);
                    break;
            } // switch $scope
        }
        
        return $eventArgs;
    }
    
    /**
     * The order that global arrays should be merged into event arguments.
     * 
     * Translates the value set by the configuration parameter
     * "parameterPrecedence" into a string where a letter corresponds
     * to the global array.
     * g = _GET (url)
     * p = _POST (form)
     * @access private
     * @return string default "gp"
     */
    private function _getRequestEventVariablesOrder()
    {
        $propertyManager = $this->_getAppManager()->getPropertyManager();
        
        $varOrder = strtolower($propertyManager->getProperty('parameterPrecedence'));
        
        switch($varOrder){
            case 'form':
            case '_post':
                $varOrder = 'gp';
                break;
                
            case 'url':
            case '_get':
                $varOrder = 'pg';
                break;
                
            case '_request':
                $varOrder = 'r';
                break;
                
            case 'variables_order':
                $varOrder = strtolower(ini_get('variables_order'));
                break;
        } // switch
        
        return $varOrder;
    }
    
    /**
     * @access private
     * @param MachII_framework_AppManager
     */
    private function _setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @access private
     * @return MachII_framework_AppManager
     */
    private function _getAppManager()
    {
        return $this->_appManager;
    }

}

?>