<?php
/**
 * @package MachII
 */
/**
 * 
 */
require_once 'MachII/util/SizedQueue.php';
require_once 'MachII/framework/ViewContext.php';

/**
 * Controls the event queue and event processing mechanism for
 * a request/event lifecycle.
 * @package MachII
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_EventContext
{
    /**
     * @var integer
     * @access private
     */
    private $_eventCount = 0;
    
    /**
     * @var MachII_framework_ViewContext
     * @access private
     */
    private $_viewContext;
    
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var SizedQueue
     * @access private
     */
    private $_eventQueue;
    
    /**
     * @var MachII_framework_EventHandler
     * @access private
     */
    private $_currentEventHandler;
    
    /**
     * @var MachII_framework_Event MachII_framework_Event or descendant
     * @access private
     */
    private $_currentEvent;
    
    /**
     * @var array associative array of event name aliases
     * @access private
     */
    private $_mappings = array();
    
    /**
     * @var string
     * @access private
     */
    private $_exceptionEventName;
    
    /**
     * @var integer really it's a string from the configuration file
     * @access private
     */
    private $_maxEvents;
    
    /**
     * @var MachII_framework_Event MachII_framework_Event or descendant
     * @access private
     */
    private $_previousEvent;
    
    
    /**
     * Initalizes the event-context.
     * 
     * @uses SizedQueue
     * @uses MachII_framework_ViewContext
     * @param MachII_framework_AppManager
     */
    public function __construct(MachII_framework_AppManager $appManager)
    {
        $this->_setAppManager($appManager);
        $this->setExceptionEventName($appManager->getPropertyManager()->getProperty('exceptionEvent'));
        $maxEvents = $appManager->getPropertyManager()->getProperty('maxEvents');
        $this->setMaxEvents($maxEvents);
        // Setup the event queue
        $this->_setEventQueue(new MachII_util_SizedQueue($maxEvents));
        
        // Setup the view context
        $this->_setViewContext(new MachII_framework_ViewContext($appManager));
    }
    
    /**
     * Announce an Event
     * 
     * Queues an event for the framework to handle.  Announces an event
     * that will cause any listeners that have registered an interest in
     * that event to execute.
     * @param string
     * @param array associative array of event arguments
     */
    public function announceEvent($eventName, $eventArgs)
    {
        try {
            // Check for event mapping
            $nextEventName = $this->getEventMapping($eventName);
            
            // Create the event
            $nextEvent = $this->_getAppManager()->getEventManager()->createEvent($nextEventName, $eventArgs);
            
            // Queue the event
            $this->_getEventQueue()->put($nextEvent);
        } catch (Exception $e) {
            $this->handleException($e,true);
        }
    }
    
    /**
     * Process Event Queue
     * 
     * Begins processing of queued events. Can only be called once.
     * 
     * Executes in the following order:
     * <ol>
     *     <li>{@link MachII_framework_Event::preProcess()}</li>
     *     <li>{@link MachII_framework_EventHandler::handleNextEvent()}</li>
     *     <li>{@link MachII_framework_Event::postProcess()}</li>
     * </ol>
     * @uses MachII_framework_Event::preProcess()
     * @uses handleNextEvent()
     * @uses MachII_framework_Event::postProcess()
     * @staticvar boolean $isProcessing
     * @throws MachII_util_MaxEventsExceededException
     */
    public function processEvents()
    {
        //echo 'processing events.<br />';
        static $isProcessing = false;
        
        if ($isProcessing == true) {
            throw new MachII_util_IsProcessingEventsException();
        }
        
        // Lock event processing
        $isProcessing = true;
        
        $appManager    = $this->_getAppManager();
        $pluginManager = $appManager->getPluginManager();
        $eventManager  = $appManager->getEventManager();
        
        // Pre-process
        $pluginManager->preProcess($this);
        
        //$GLOBALS['vard']->display(debug_backtrace());
        // Process
        //echo 'has more events: ',var_dump($this->hasMoreEvents()),'<br />';
        //echo 'event count: ',$this->getEventCount(),'<br />';
        //echo 'under max events: ',var_dump(($this->getEventCount() < $this->getMaxEvents())),'<br />';
        while ($this->hasMoreEvents()
        && ($this->getEventCount() < $this->getMaxEvents())) {
            //echo 'handling event.<br />';
            $this->_handleNextEvent();
        }
        
        // If there are still events in the queue after done processing, then throw an exception.
        if (!$this->_getEventQueue()->isEmpty()) {
            $e = new MachII_util_MaxEventsExceededException(array('maxEventsNum' => $this->getMaxEvents()));
            $this->handleException($e, true);
        }
        
        // Post-process
        $pluginManager->postProcess($this);
        
        // Unlock event processing
        $isProcessing = false;
    }  
     
    /**
     * @param MachII_framework_Event or descendant
     */
    public function setCurrentEvent(MachII_framework_Event $currentEvent)
    {
        $this->_currentEvent = $currentEvent;
    }
    
    /**
     * @return MachII_framework_Event or descendant
     */
    public function getCurrentEvent()
    {
        return $this->_currentEvent;
    }
    
    /**
     * @return boolean
     */
    public function hasCurrentEvent()
    {
        return $this->_currentEvent instanceof MachII_framework_Event;
    }
    
    /**
     * Peeks at the next event in the queue.
     * @return MachII_framework_Event MachII_framework_Event or descendant
     */
    public function getNextEvent()
    {
        return $this->_getEventQueue()->peek();
    }
    
    /**
     * Returns whether there is another event in the queue.
     * 
     * This is an alias for {@link hasMoreEvents()}.
     * @return boolean
     */
    public function hasNextEvent()
    {
        return $this->hasMoreEvents();
    }
    
    /**
     * @access private
     * @param MachII_framework_Event MachII_framework_Event or descendant
     */
    private function _setPreviousEvent(MachII_framework_Event $previousEvent)
    {
        $this->_previousEvent = $previousEvent;
    }
    
    /**
     * Returns the previous handled event.
     * @return MachII_framework_Event MachII_framework_Event or descendant
     */
    public function getPreviousEvent()
    {
        return $this->_previousEvent;
    }
    
    /**
     * Returns whether or not {@link getPreviousEvent()} can be called
     * to return an event.
     * @return boolean
     */
    public function hasPreviousEvent()
    {
        return isset($this->_previousEvent);
    }
        
    /**
     * Returns whether there is another event in the queue.
     * 
     * This has an alias called {@link hasNextEvent()}.
     * return boolean
     */
    public function hasMoreEvents()
    {
        return !$this->_getEventQueue()->isEmpty();
    }
    
    /**
     * @param string
     * @param string
     */
    public function setEventMapping($eventName, $mappingName)
    {
        $this->_mappings[$eventName] = $mappingName;
    }
    
    /**
     * @return string
     */
    public function getEventMapping($eventName)
    {
        $eventName = strtolower($eventName);
        
        return (isset($this->_mappings[$eventName]))?
                $this->_mappings[$eventName]:
                $eventName;
    }
    
    /**
     * 
     */
    public function clearEventMappings()
    {
        $this->_mappings = array();
    }
    
    /**
     * @param Exception
     * @return Exception
     */
    public function createException(Exception $e)
    {
        return $e;
    }
    
    /**
     * Execute the registered exception event
     * 
     * The name of the exception event is set in the configuration file.
     * @param MachII_util_Exception
     * @param boolean
     */
    public function handleException(MachII_util_Exception $exception, $clearEventQueue = true)
    {
        try {
            $appManager    = $this->_getAppManager();
            $eventManager  = $appManager->getEventManager();
            $pluginManager = $appManager->getPluginManager();
            
            $nextEventName = $this->getEventMapping($this->getExceptionEventName());
            
            $exceptionEvent = $eventManager->createEvent($nextEventName);
            $newException   = $exceptionEvent;
            $exceptionEvent->setArg('exception', $exception);
            if ($this->hasCurrentEvent()) {
                $exceptionEvent->setArg('exceptionEvent', $this->getCurrentEvent());
            }
            
            $pluginManager->handleException($this, $exception);
            
            if ($clearEventQueue) {
                $this->clearEventQueue();
            }
            
            $this->_getEventQueue()->put($exceptionEvent);
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Display an event's views
     * 
     * Executes in the following order:
     * <ol>
     *     <li>{@link MachII_framework_Plugin::preView()}</li>
     *     <li>{@link MachII_framework_ViewContext::displayView()}</li>
     *     <li>{@link MachII_framework_Plugin::postView()}</li>
     * </ol>
     * @uses MachII_framework_Plugin::preView()
     * @uses MachII_framework_ViewContext::displayView()
     * @uses MachII_framework_Plugin::postView()}
     * @param MachII_framework_Event or descendant
     * @param string
     * @param string
     * @param boolean
     */
    public function displayView(MachII_framework_Event $event, $viewName, $contentKey, $append = false)
    {//echo 'EventContext->displayView - $viewName: ',$viewName,'<br />';
        $pluginManager = $this->_getAppManager()->getPluginManager();
        
        // Pre-view
        $pluginManager->preView($this);
        
        // Display view
        $this->_getViewContext()->displayView($event, $viewName, $contentKey, $append);
        
        // Post-view
        $pluginManager->postView($this);
    }
    
    /**
     * @param integer
     */
    public function setMaxEvents($maxEvents)
    {
        $this->_maxEvents = $maxEvents;
    }
    
    /**
     * @return integer
     */
    public function getMaxEvents()
    {
        return $this->_maxEvents;
    }
    
    /**
     * @param string
     */
    public function setExceptionEventName($exceptionEventName)
    {
        $this->_exceptionEventName = strtolower($exceptionEventName);
    }
    
    /**
     * @return string
     */
    public function getExceptionEventName()
    {
        return $this->_exceptionEventName;
    }
    
    /**
     * Clears the event queue.
     */
    public function clearEventQueue()
    {
        $this->_getEventQueue()->clear();
    }
    
    /**
     * Returns the number of events that have been processed for this context.
     * 
     * @return integer
     */
    public function getEventCount()
    {
        return $this->_eventCount;
    }    
    
    /**
     * Execute the next event in the queue
     * 
     * If the event returns an error, pass it to the exception handler.
     * @access private
     * @uses getNextEvent()
     * @uses _handleEvent()
     * @uses handleException()
     */
    private function _handleNextEvent()
    {
        try {
            $this->_incrementEventCount();
            $this->_handleEvent($this->_getEventQueue()->get());
        } catch (MachII_util_AbortEventException $e) {
            // Do nothing, just continue processing
        } catch (Exception $e) {
            $this->handleException($e, true);
        }
    }
    
    /**
     * Execute an event.
     * 
     * Executes in the following order:
     * <ol>
     *     <li>{@link MachII_framework_Plugin::preEvent()}</li>
     *     <li>{@link MachII_framework_Event::preInvoke()}</li>
     *     <li>{@link MachII_framework_EventHandler::handleEvent()}</li>
     *     <li>{@link MachII_framework_Event::postInvoke()}</li>
     *     <li>{@link MachII_framework_Plugin::postEvent()}</li>
     * </ol>
     * @access private
     * @global MachII_framework_Event Sets $GLOBALS['event'] to the current
     *         MachII_framework_Event or descendant.
     * @uses MachII_framework_Plugin::preEvent()
     * @uses MachII_framework_Event::preInvoke()
     * @uses MachII_framework_EventHandler::handleEvent()
     * @uses MachII_framework_Event::postInvoke()
     * @uses MachII_framework_Plugin::postEvent()
     * @param MachII_framework_Event MachII_framework_Event or descendant
     */
    private function _handleEvent(MachII_framework_Event $event)
    {
        if ($this->hasCurrentEvent()) {
            $this->_setPreviousEvent($this->getCurrentEvent());
        }
        $this->setCurrentEvent($event);
        $GLOBALS['event'] =& $event;
        
        $eventName = $event->getName();
        
        $appManager = $this->_getAppManager();
        
        $pluginManager = $appManager->getPluginManager();
        
        $eventHandler = $appManager->getEventManager()->getEventHandler($eventName);
        $this->_setCurrentEventHandler($eventHandler);
        
        // Pre-invoke
        $pluginManager->preEvent($this);
        
        // Invoke event
        $eventHandler->handleEvent($event, $this);
        
        // Post invoke
        $pluginManager->postEvent($this);
        
        $this->clearEventMappings();
    }
    
    /**
     * @access private
     * @param MachII_framework_EventHandler
     */
    private function _setCurrentEventHandler(MachII_framework_EventHandler $currentEventHandler)
    {
        $this->_currentEventHandler = $currentEventHandler;
    }
    
    /**
     * @access private
     * @return MachII_framework_EventHandler
     */
    private function _getCurrentEventHandler()
    {
        return $this->_currentEventHandler;
    }
    
    /**
     * @access private
     */
    private function _incrementEventCount()
    {
        $this->_eventCount++;
    }
    
    /**
     * @access private
     * @param MachII_framework_AppManager
     */
    private function _setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @access private
     * @return MachII_framework_AppManager
     */
    private function _getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param MachII_framework_ViewContext
     */
    private function _setViewContext(MachII_framework_ViewContext $viewContext)
    {
        $this->_viewContext = $viewContext;
    }
    
    /**
     * @return MachII_framework_ViewContext
     */
    private function _getViewContext()
    {
        return $this->_viewContext;
    }
    
    /**
     * @access private
     * @param MachII_util_SizedQueue
     */
    private function _setEventQueue(MachII_util_SizedQueue $eventQueue)
    {
        $this->_eventQueue = $eventQueue;
    }
    
    /**
     * @access private
     */
    private function _getEventQueue()
    {
        return $this->_eventQueue;
    }

}

?>