<?php
/**
 * @package MachII
 */
/**
 * Load base plugin class
 */
require_once 'MachII/framework/Plugin.php';
/**
 * Manages registered plugins.
 * @package MachII
 * @author Ben Edwards <ben@ben-edwards.com>
 * @author Alan Richmond <alan@aardwolfweb.com>
 */
class MachII_framework_PluginManager
{
    /**
     * @var MachII_framework_AppManager
     * @access private
     */
    private $_appManager;
    
    /**
     * @var array associative array of MachII_framework_Plugin.
     * @access private
     */
    private $_plugins = array();
    
    /**
     * @var array associative array of MachII_framework_Plugin.
     * @access private
     */
    private $_pluginsArray = array();
    
    
    /**
     * @param SimpleXMLElement
     * @param MachII_framework_AppManager
     */
    public function __construct(SimpleXMLElement $configXml, MachII_framework_AppManager $appManager)
    {
        $this->setAppManager($appManager);
        $plugins = $configXml->xpath('//plugins/plugin');
        if (empty($plugins)) {
            return;
        }
        
        foreach ($plugins as $plugin) {
            $parameters = isset($plugin->parameters->parameter)?
                $plugin->parameters->parameter:
                array();
                
            $pluginParams = array();
            foreach ($parameters as $param) {
                $pluginParams[(string) $param['name']] = (string) $param['value'];
            }
            
            $pluginType = MachII_util_ClassLoader::load((string) $plugin['type']);
            $pluginObj = new $pluginType($appManager, $pluginParams);
            
            $this->addPlugin((string) $plugin['name'], $pluginObj);
        }
    }
    
    /**
     * Configure all of the plugins.
     */
    public function configure()
    {
        for ($key = 0, $cnt = count($this->_pluginsArray); $key < $cnt; $key++) {
            $this->_pluginsArray[$key]->configure();
        }
    }
    
    /**
     * @param string
     * @param MachII_framework_Plugin descendent
     */
    public function addPlugin($pluginName, MachII_framework_Plugin $plugin)
    {
        $this->_plugins[$pluginName] = $plugin;
        $this->_pluginsArray[] = $plugin;
    }
    
    /**
     * @param string
     * @return MachII_framework_Plugin descendent
     */
    public function getPlugin($pluginName)
    {
        return $this->_plugins[$pluginName];
    }
    
    /**
     * @param MachII_framework_AppManager
     */
    public function setAppManager(MachII_framework_AppManager $appManager)
    {
        $this->_appManager = $appManager;
    }
    
    /**
     * @return MachII_framework_AppManager
     */
    public function getAppManager()
    {
        return $this->_appManager;
    }
    
    /**
     * @param string method name of a plugin point
     * @param MachII_framework_EventContext
     */
    public function executePluginPoint($pluginPoint, MachII_framework_EventContext $eventContext)
    {
        for ($key = 0, $cnt = count($this->_pluginsArray); $key < $cnt; $key++) {
            $this->_pluginsArray[$key]->$pluginPoint($eventContext);
        }
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function preProcess(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('preProcess', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function preEvent(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('preEvent', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function postEvent(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('postEvent', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function preView(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('preView', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function postView(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('postView', $eventContext);
    }
    
    /**
     * @param MachII_framework_EventContext
     * @uses executePluginPoint()
     */
    public function postProcess(MachII_framework_EventContext $eventContext)
    {
        $this->executePluginPoint('postProcess', $eventContext);
    }
    
    /**
     * Executes handleException() for each registered plugin.
     * @param MachII_framework_EventContext
     * @param MachII_framework_Exception
     * @see MachII_framework_EventContext::handleException()
     */
    public function handleException(MachII_framework_EventContext $eventContext, MachII_util_Exception $exception)
    {
        for ($key = 0, $cnt = count($this->_pluginsArray); $key < $cnt; $key++) {
            $this->getPlugin($key)->handleException($eventContext, $exception);
        }
    }
}
 
?>
